﻿using System;
using System.Linq;

namespace TestLibrary
{
    /// <summary>
    /// 평균 피크 공급자
    /// </summary>
    public class AveragePeakProvider : PeakProvider
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 스케일
        /// </summary>
        private readonly float scale;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - AveragePeakProvider(scale)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="scale">스케일</param>
        public AveragePeakProvider(float scale)
        {
            this.scale = scale;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 다음 피크 구하기 - GetNextPeak()

        /// <summary>
        /// 다음 피크 구하기
        /// </summary>
        /// <returns>다음 피크</returns>
        public override PeakInfo GetNextPeakInfo()
        {
            int sampleCountRead = Provider.Read(ReadBuffer, 0, ReadBuffer.Length);

            float summary = (sampleCountRead == 0) ? 0 : ReadBuffer.Take(sampleCountRead).Select(sample => Math.Abs(sample)).Sum();

            float average = summary / sampleCountRead;
            
            return new PeakInfo(average * (0 - scale), average * scale);
        }

        #endregion
    }
}