﻿using System;

using NAudio.Wave;

namespace TestLibrary
{
    /// <summary>
    /// 데시벨 피크 공급자
    /// </summary>
    public class DecibelPeakProvider : IPeakProvider
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 소스 공급자
        /// </summary>
        private readonly IPeakProvider sourceProvider;

        /// <summary>
        /// 동적 범위
        /// </summary>
        private readonly double dynamicRange;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - DecibelPeakProvider(sourceProvider, dynamicRange)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="sourceProvider">소스 공급자</param>
        /// <param name="dynamicRange">동적 범위</param>
        public DecibelPeakProvider(IPeakProvider sourceProvider, double dynamicRange)
        {
            this.sourceProvider = sourceProvider;
            this.dynamicRange   = dynamicRange;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 초기화하기 - Initialize(provider, sampleCountPerPixel)

        /// <summary>
        /// 초기화하기
        /// </summary>
        /// <param name="provider">샘플 공급자 인터페이스</param>
        /// <param name="sampleCountPerPixel">픽셀당 샘플 수</param>
        public void Initialize(ISampleProvider provider, int sampleCountPerPixel)
        {
            throw new NotImplementedException();
        }

        #endregion
        #region 다음 피크 정보 구하기 - GetNextPeakInfo()

        /// <summary>
        /// 다음 피크 정보 구하기
        /// </summary>
        /// <returns>다음 피크 정보</returns>
        public PeakInfo GetNextPeakInfo()
        {
            PeakInfo peakInfo = this.sourceProvider.GetNextPeakInfo();

            double maximumDecibel = 20 * Math.Log10(peakInfo.Maximum);

            if(maximumDecibel < 0 - dynamicRange)
            {
                maximumDecibel = 0 - dynamicRange;
            }

            float linear = (float)((dynamicRange + maximumDecibel) / dynamicRange);

            return new PeakInfo(0 - linear, linear);
        }

        #endregion
    }
}