﻿using NAudio.Wave;

namespace TestLibrary
{
    /// <summary>
    /// 피크 공급자
    /// </summary>
    public abstract class PeakProvider : IPeakProvider
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 공급자 - Provider

        /// <summary>
        /// 공급자
        /// </summary>
        protected ISampleProvider Provider { get; private set; }

        #endregion
        #region 피크당 샘플 수 - SampleCountPerPeak

        /// <summary>
        /// 피크당 샘플 수
        /// </summary>
        protected int SampleCountPerPeak { get; private set; }

        #endregion
        #region 읽기 버퍼 - ReadBuffer

        /// <summary>
        /// 읽기 버퍼
        /// </summary>
        protected float[] ReadBuffer { get; private set; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 초기화하기 - Initialize(provider, sampleCountPerPeak)

        /// <summary>
        /// 초기화하기
        /// </summary>
        /// <param name="provider">샘플 공급자</param>
        /// <param name="sampleCountPerPeak">피크당 샘플 수</param>
        public void Initialize(ISampleProvider provider, int sampleCountPerPeak)
        {
            Provider           = provider;
            SampleCountPerPeak = sampleCountPerPeak;
            ReadBuffer         = new float[sampleCountPerPeak];
        }

        #endregion
        #region 다음 피크 정보 구하기 - GetNextPeakInfo()

        /// <summary>
        /// 다음 피크 정보 구하기
        /// </summary>
        /// <returns>다음 피크 정보</returns>
        public abstract PeakInfo GetNextPeakInfo();

        #endregion
    }
}