﻿using System;

namespace TestLibrary
{
    /// <summary>
    /// RMS 피크 공급자
    /// </summary>
    public class RMSPeakProvider : PeakProvider
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 블럭 크기
        /// </summary>
        private readonly int blockSize;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - RMSPeakProvider(blockSize)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="blockSize">블럭 크기</param>
        public RMSPeakProvider(int blockSize)
        {
            this.blockSize = blockSize;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 다음 피크 정보 구하기 - GetNextPeakInfo()

        /// <summary>
        /// 다음 피크 정보 구하기
        /// </summary>
        /// <returns>다음 피크 정보</returns>
        public override PeakInfo GetNextPeakInfo()
        {
            int sampleCountRead = Provider.Read(ReadBuffer, 0, ReadBuffer.Length);

            float maximum = 0.0f;

            for(int x = 0; x < sampleCountRead; x += blockSize)
            {
                double total = 0.0;

                for(int y = 0; y < blockSize && x + y < sampleCountRead; y++)
                {
                    total += ReadBuffer[x + y] * ReadBuffer[x + y];
                }

                float rms = (float) Math.Sqrt(total / blockSize);

                maximum = Math.Max(maximum, rms);
            }

            return new PeakInfo(0 -maximum, maximum);
        }

        #endregion
    }
}