﻿using System;

namespace TestLibrary
{
    /// <summary>
    /// 샘플링 피크 공급자
    /// </summary>
    public class SamplingPeakProvider : PeakProvider
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 샘플 간격
        /// </summary>
        private readonly int sampleInterval;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - SamplingPeakProvider(sampleInterval)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="sampleInterval">샘플 간격</param>
        public SamplingPeakProvider(int sampleInterval)
        {
            this.sampleInterval = sampleInterval;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 다음 피크 정보 구하기 - GetNextPeakInfo()

        /// <summary>
        /// 다음 피크 정보 구하기
        /// </summary>
        /// <returns>다음 피크 정보</returns>
        public override PeakInfo GetNextPeakInfo()
        {
            int sampleCountRead = Provider.Read(ReadBuffer,0,ReadBuffer.Length);

            float minimum = 0.0f;
            float maximum = 0.0f;

            for(int x = 0; x < sampleCountRead; x += sampleInterval)
            {
                minimum = Math.Min(minimum, ReadBuffer[x]);
                maximum = Math.Max(maximum, ReadBuffer[x]);
            }

            return new PeakInfo(minimum,maximum);
        }

        #endregion
    }
}