using System;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace TestLibrary
{
    /// <summary>
    ///  Ŭ  
    /// </summary>
    public class SoundCloudOriginalSetting : WaveFormRendererSetting
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// ֱ  
        /// </summary>
        private int lastTopHeight;

        /// <summary>
        /// ֱ ϴ 
        /// </summary>
        private int lastBottomHeight;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  ũ  - TopPeakPen

        /// <summary>
        ///  ũ 
        /// </summary>
        public override Pen TopPeakPen
        {
            get
            {
                if(base.TopPeakPen == null || this.lastTopHeight != TopHeight)
                {
                    base.TopPeakPen = CreateGradientPen(TopHeight, Color.FromArgb(120, 120, 120), Color.FromArgb(50, 50, 50));

                    this.lastTopHeight = TopHeight;
                }

                return base.TopPeakPen;
            }
            set
            {
                base.TopPeakPen = value;
            }
        }

        #endregion
        #region ϴ ũ  - BottomPeakPen

        /// <summary>
        /// ϴ ũ 
        /// </summary>
        public override Pen BottomPeakPen 
        {
            get
            {
                if(base.BottomPeakPen == null || this.lastBottomHeight != BottomHeight || this.lastTopHeight != TopHeight)
                {
                    base.BottomPeakPen = CreateSoundCloudBottomPen(TopHeight, BottomHeight);

                    this.lastTopHeight    = TopHeight;
                    this.lastBottomHeight = BottomHeight;
                }

                return base.BottomPeakPen;
            }

            set
            {
                base.BottomPeakPen = value;
            }
        }

        #endregion
        #region ϴ   - BottomSpacerPen

        /// <summary>
        /// ϴ  
        /// </summary>
        public override Pen BottomSpacerPen
        {
            get
            {
                throw new InvalidOperationException("No spacer pen required");
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  - SoundCloudOriginalSetting()

        /// <summary>
        /// 
        /// </summary>
        public SoundCloudOriginalSetting()
        {
            PixelCountPerPeak = 1;
            SpacerPixelCount  = 0;
            BackgroundColor   = Color.White;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region  Ŭ ϴ  ϱ - CreateSoundCloudBottomPen(topHeight, bottomHeight)

        /// <summary>
        ///  Ŭ ϴ  ϱ
        /// </summary>
        /// <param name="topHeight"></param>
        /// <param name="bottomHeight"></param>
        /// <returns> Ŭ ϴ </returns>
        private Pen CreateSoundCloudBottomPen(int topHeight, int bottomHeight)
        {
            LinearGradientBrush brush = new LinearGradientBrush
            (
                new Point(0, topHeight),
                new Point(0, topHeight + bottomHeight), 
                Color.FromArgb(16, 16, 16),
                Color.FromArgb(150, 150, 150)
            );

            ColorBlend colorBlend = new ColorBlend(3);

            colorBlend.Colors[0]    = Color.FromArgb(16, 16, 16);
            colorBlend.Colors[1]    = Color.FromArgb(142, 142, 142);
            colorBlend.Colors[2]    = Color.FromArgb(150, 150, 150);
            colorBlend.Positions[0] = 0;
            colorBlend.Positions[1] = 0.1f;
            colorBlend.Positions[2] = 1.0f;

            brush.InterpolationColors = colorBlend;

            return new Pen(brush);
        }

        #endregion
    }
}