﻿using System;
using System.Drawing;
using System.Threading.Tasks;
using System.Windows.Forms;

using NAudio.Wave;

using TestLibrary;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 오디오 파일 경로
        /// </summary>
        private string audioFilePath;

        /// <summary>
        /// 이미지 파일 경로
        /// </summary>
        private string imageFilePath;

        /// <summary>
        /// 웨이브 폼 렌더러
        /// </summary>
        private readonly WaveFormRenderer render;

        /// <summary>
        /// 웨이브 폼 렌더러 설정
        /// </summary>
        private readonly WaveFormRendererSetting setting;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.render = new WaveFormRenderer();

            Color topSpacerColor1 = Color.FromArgb(64, 83 , 22 , 3  );
            Color topSpacerColor2 = Color.FromArgb(64, 224, 224, 224);

            #region 설정 항목을 설정한다.

            StandardWaveFormRendererSetting setting1 = new StandardWaveFormRendererSetting()
            {
                Name = "Standard"
            };

            SoundCloudOriginalSetting setting2 = new SoundCloudOriginalSetting()
            {
                Name = "SoundCloud Original"
            };

            SoundCloudBlockWaveFormSetting setting3 = new SoundCloudBlockWaveFormSetting
            (
                Color.FromArgb(102, 102, 102),
                Color.FromArgb(103, 103, 103),
                Color.FromArgb(179, 179, 179),
                Color.FromArgb(218, 218, 218)
            )
            {
                Name = "SoundCloud Light Blocks"
            };

            SoundCloudBlockWaveFormSetting setting4 = new SoundCloudBlockWaveFormSetting
            (
                Color.FromArgb(52 , 52 , 52 ),
                Color.FromArgb(55 , 55 , 55 ),
                Color.FromArgb(154, 154, 154),
                Color.FromArgb(204, 204, 204)
            )
            {
                Name = "SoundCloud Darker Blocks"
            };

            SoundCloudBlockWaveFormSetting setting5 = new SoundCloudBlockWaveFormSetting
            (
                Color.FromArgb(255, 76 , 0  ),
                Color.FromArgb(255, 52 , 2  ),
                Color.FromArgb(255, 171, 141),
                Color.FromArgb(255, 213, 199)
            )
            {
                Name = "SoundCloud Orange Blocks"
            };

            SoundCloudBlockWaveFormSetting setting6 = new SoundCloudBlockWaveFormSetting
            (
                Color.FromArgb(196, 197, 53, 0 ),
                topSpacerColor1,
                Color.FromArgb(196, 79 , 26, 0 ),
                Color.FromArgb(64 , 79 , 79, 79)
            ) 
            { 
                Name                        = "SoundCloud Orange Transparent Blocks", 
                PixelCountPerPeak           = 2, 
                SpacerPixelCount            = 1,
                TopSpacerGradientStartColor = topSpacerColor1,
                BackgroundColor             = Color.Transparent
            };

            SoundCloudBlockWaveFormSetting setting7 = new SoundCloudBlockWaveFormSetting
            (
                Color.FromArgb(196, 224, 225, 224),
                topSpacerColor2,
                Color.FromArgb(196, 128, 128, 128),
                Color.FromArgb(64 , 128, 128, 128)
            )
            {
                Name                        = "SoundCloud Gray Transparent Blocks",
                PixelCountPerPeak           = 2,
                SpacerPixelCount            = 1,
                TopSpacerGradientStartColor = topSpacerColor2,
                BackgroundColor             = Color.Transparent
            };

            #endregion

            this.setting = setting1;

            #region 피크 계산 전략 콤보 박스를 설정한다.

            this.peakCalculationStrategyComboBox.Items.Add("Max Absolute Value");
            this.peakCalculationStrategyComboBox.Items.Add("Max Rms Value"     );
            this.peakCalculationStrategyComboBox.Items.Add("Sampled Peaks"     );
            this.peakCalculationStrategyComboBox.Items.Add("Scaled Average"    );

            this.peakCalculationStrategyComboBox.SelectedIndex = 0;

            #endregion
            #region 렌더링 스타일 콤보 박스를 설정한다.

            this.renderingStyleComboBox.DisplayMember = "Name";

            this.renderingStyleComboBox.DataSource = new WaveFormRendererSetting[]
            {
                setting1, 
                setting2, 
                setting3, 
                setting4, 
                setting5, 
                setting6,
                setting7
            };

            this.renderingStyleComboBox.SelectedIndex = 5;

            #endregion
            #region 상단 색상 버튼을 설정한다.

            this.topColorButton.BackColor = this.setting.TopPeakPen.Color;

            #endregion
            #region 하단 색상 버튼을 설정한다.

            this.bottomColorButton.BackColor = this.setting.BottomPeakPen.Color;

            #endregion
            #region Rendering 레이블을 설정한다.

            this.renderingLabel.Visible = false;

            #endregion
            #region 이벤트를 설정한다.

            this.loadButton.Click                                     += loadButton_Click;
            this.peakCalculationStrategyComboBox.SelectedIndexChanged += peakCalculationStrategyComboBox_SelectedIndexChanged; ;
            this.decibelCheckBox.CheckedChanged                       += decibelCheckBox_CheckedChanged;
            this.renderingStyleComboBox.SelectedIndexChanged          += renderingStyleComboBox_SelectedIndexChanged;
            this.topColorButton.Click                                 += colorButton_Click;
            this.bottomColorButton.Click                              += colorButton_Click;
            this.loadBackgroundImageButton.Click                      += loadBackgroundImageButton_Click;
            this.refreshImageButton.Click                             += refreshImageButton_Click;
            this.saveButton.Click                                     += saveButton_Click;

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 오디오 파일 로드 버튼 클릭시 처리하기 - loadButton_Click(sender, e)

        /// <summary>
        /// 오디오 파일 로드 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void loadButton_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();

            openFileDialog.Filter = "MP3 파일|*.mp3|WAV 파일|*.wav";

            if(openFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                this.audioFilePath = openFileDialog.FileName;

                RenderWaveform();
            }
        }

        #endregion
        #region 피크 계산 전략 콤보 박스 선택 인덱스 변경시 처리하기 - peakCalculationStrategyComboBox_SelectedIndexChanged(sender, e)

        /// <summary>
        /// 피크 계산 전략 콤보 박스 선택 인덱스 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void peakCalculationStrategyComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            RenderWaveform();
        }

        #endregion
        #region 데시빌 체크 박스 체크 변경시 처리하기 - decibelCheckBox_CheckedChanged(sender, e)

        /// <summary>
        /// 데시빌 체크 박스 체크 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void decibelCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            RenderWaveform();
        }

        #endregion
        #region 렌더링 스타일 콤보 박스 선택 인덱스 변경시 처리하기 - renderingStyleComboBox_SelectedIndexChanged(sender, e)

        /// <summary>
        /// 렌더링 스타일 콤보 박스 선택 인덱스 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void renderingStyleComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            RenderWaveform();
        }

        #endregion
        #region 색상 버튼 클릭시 처리하기 - colorButton_Click(sender, e)

        /// <summary>
        /// 색상 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void colorButton_Click(object sender, EventArgs e)
        {
            Button button = sender as Button;

            ColorDialog colorDialog = new ColorDialog();

            colorDialog.Color = button.BackColor;

            if(colorDialog.ShowDialog(this) == DialogResult.OK)
            {
                button.BackColor = colorDialog.Color;

                this.setting.TopPeakPen    = new Pen(this.topColorButton.BackColor   );
                this.setting.BottomPeakPen = new Pen(this.bottomColorButton.BackColor);

                RenderWaveform();
            }
        }

        #endregion
        #region 배경 이미지 로드 버튼 클릭시 처리하기 - loadBackgroundImageButton_Click(sender, e)

        /// <summary>
        /// 배경 이미지 로드 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void loadBackgroundImageButton_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();

            openFileDialog.Filter = "이미지 파일|*.bmp;*.png;*.jpg";

            if(openFileDialog.ShowDialog() == DialogResult.OK)
            {
                this.imageFilePath = openFileDialog.FileName;

                RenderWaveform();
            }
        }

        #endregion
        #region 이미지 갱신 버튼 클릭시 처리하기 - refreshImageButton_Click(sender, e)

        /// <summary>
        /// 이미지 갱신 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void refreshImageButton_Click(object sender, EventArgs e)
        {
            RenderWaveform();
        }

        #endregion
        #region 저장 버튼 클릭시 처리하기 - saveButton_Click(sender, e)

        /// <summary>
        /// 저장 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void saveButton_Click(object sender, EventArgs e)
        {
            SaveFileDialog saveFileDialog = new SaveFileDialog();

            saveFileDialog.Filter = "PNG 파일|*.png";

            if(saveFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                this.pictureBox.Image.Save(saveFileDialog.FileName);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 웨이브 폼 렌더러 설정 구하기 - GetWaveFormRendererSetting()

        /// <summary>
        /// 웨이브 폼 렌더러 설정 구하기
        /// </summary>
        /// <returns>웨이브 폼 렌더러 설정</returns>
        private WaveFormRendererSetting GetWaveFormRendererSetting()
        {
            WaveFormRendererSetting setting = (WaveFormRendererSetting)this.renderingStyleComboBox.SelectedItem;

            setting.TopHeight    = (int)this.topHeightNumericUpDown.Value;
            setting.BottomHeight = (int)this.bottomHeightNumericUpDown.Value;
            setting.Width        = (int)this.imageWidthNumericUpDown.Value;
            setting.DecibelScale = this.decibelCheckBox.Checked;

            return setting;
        }

        #endregion
        #region 피크 공급자 구하기 - GetPeakProvider()

        /// <summary>
        /// 피크 공급자 구하기
        /// </summary>
        /// <returns>피크 공급자</returns>
        private IPeakProvider GetPeakProvider()
        {
            switch(this.peakCalculationStrategyComboBox.SelectedIndex)
            {
                case 0  : return new MaximumPeakProvider();
                case 1  : return new RMSPeakProvider((int)this.blockSizeNumericUpDown.Value);
                case 2  : return new SamplingPeakProvider((int)this.blockSizeNumericUpDown.Value);
                case 3  : return new AveragePeakProvider(4);
                default : throw  new InvalidOperationException("Unknown calculation strategy");
            }
        }

        #endregion
        #region 렌더링 스레딩 처리하기 - ProcessRenderingThread(peakProvider, setting)

        /// <summary>
        /// 렌더링 스레딩 처리하기
        /// </summary>
        /// <param name="peakProvider">피크 공급자</param>
        /// <param name="setting">설정</param>
        private void ProcessRenderingThread(IPeakProvider peakProvider, WaveFormRendererSetting setting)
        {
            Image image = null;

            try
            {
                using(AudioFileReader reader = new AudioFileReader(this.audioFilePath))
                {
                    image = this.render.Render(reader, peakProvider, setting);
                }
            }
            catch(Exception exception)
            {
                MessageBox.Show(exception.Message);
            }

            BeginInvoke
            (
                (Action)
                (
                    () =>
                    {
                        this.renderingLabel.Visible = false;

                        this.pictureBox.Image = image;

                        Enabled = true;
                    }
                )
            );
        }

        #endregion
        #region 웨이브 폼 렌더링하기 - RenderWaveform()

        /// <summary>
        /// 웨이브 폼 렌더링하기
        /// </summary>
        private void RenderWaveform()
        {
            if(this.audioFilePath == null)
            {
                return;
            }

            WaveFormRendererSetting setting = GetWaveFormRendererSetting();

            if(this.imageFilePath != null)
            {
                setting.BackgroundImage = new Bitmap(this.imageFilePath);
            }

            this.pictureBox.Image = null;

            this.renderingLabel.Visible = true;

            Enabled = false;

            IPeakProvider peakProvider = GetPeakProvider();

            Task.Factory.StartNew(() => ProcessRenderingThread(peakProvider, setting));
        }

        #endregion
    }
}